//
//  test_rapidPiPoTools.cpp
//  Unit tests for rapidPiPoTools
//

#ifndef CATCH_CONFIG_MAIN
#define CATCH_CONFIG_MAIN
#endif

#ifdef RTA_USE_VECLIB
#undef RTA_USE_VECLIB
#endif

#include <unistd.h>
#include "catch.hpp"
#include "signalProcessing.h"

#define MAX_PATH_SIZE 256

//static

//================================== GMM =====================================//

SCENARIO("Test rapidPiPoHost", "[signalProcessing]")
{
    GIVEN("A rapidPiPoHost class with a rapidPiPo chain and an audio file")
    {
        maxiSample buffer;

        // in XCode this gives the path to DerivedData folder
        // char pathStrBuf[MAX_PATH_SIZE];
        // char *cwd = getcwd(pathStrBuf, sizeof(pathStrBuf));
        // std::cout << std::string(cwd) << std::endl;
        
        // but here we just add the file to the Copy File(s) Build Phase
        buffer.load("./DnB-loop-175BPM.wav", 0);
        // ( source : http://freesound.org/people/yewbic/sounds/40107/ )

        buffer.reset(); // (no real need to do this here)
        
        //====================================================================//

        // instantiate PiPo related classes here :
        rapidPiPoOwner pipoHost; // -> this class is located in rapidPiPoTools
        
        // if we want to add some custom PiPos to our collection :
        // #include "myCustomPiPo.h"
        // PiPoCollection::addToCollection("myCustomPiPo", new PiPoCreator<myCustomPiPo>);
        // now we can write :
        // pipoHost.setChain("myCustomPiPo");

#include "PiPoMaximChroma.h"
        // this one is not part of the default collection :
        PiPoCollection::addToCollection("chroma", new PiPoCreator<PiPoMaximChroma>);
        
        pipoHost.setChain("slice:fft:sum:scale:onseg");
        
        pipoHost.setAttr("slice.size", 1024);
        pipoHost.setAttr("slice.hop", 256);
        pipoHost.setAttr("slice.norm", "power");
        pipoHost.setAttr("fft.mode", "power");
        pipoHost.setAttr("fft.weighting", "itur468");
        pipoHost.setAttr("scale.inmin", 1.);
        pipoHost.setAttr("scale.inmax", 10.);
        pipoHost.setAttr("scale.outmin", 0.);
        pipoHost.setAttr("scale.outmax", 10.);
        pipoHost.setAttr("scale.func", "log");
        pipoHost.setAttr("scale.base", 10.);
        pipoHost.setAttr("onseg.duration", 1.);
        pipoHost.setAttr("onseg.min", 0.);
        pipoHost.setAttr("onseg.max", 1.);
        pipoHost.setAttr("onseg.mean", 0.);
        pipoHost.setAttr("onseg.stddev", 0.);
        pipoHost.setAttr("onseg.startisonset", 1.);
        pipoHost.setAttr("onseg.threshold", 9.);
        pipoHost.setAttr("onseg.offthresh", -120.);
        
        // set another chain :
        // pipoHost.setChain("chroma");
        
        WHEN("file is processed")
        {

            std::vector<std::string> labels;
            pipoHost.streamAttributes(true, 44100, 0, 1, 1, labels, false, 0, 1);
            
            float value;
            for (unsigned int i = 0; i < buffer.length; ++i) {
                value = buffer.play();
                pipoHost.frames((double)i, 0.0, &value, 1, 1);
            }

            THEN("compare results of actual processing and same file processed in another envrironment")
            {
                // compare the results of the processing

                REQUIRE(true);
            }
        }
    }
}
